/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

package org.ow2.orchestra.designer.control {
import com.adobe.cairngorm.CairngormError;
import com.adobe.cairngorm.CairngormMessageCodes;
import com.adobe.cairngorm.control.FrontController;

import org.ow2.orchestra.designer.commands.ChangeAccessSettingsProcessCMD;
import org.ow2.orchestra.designer.commands.CloseProcessCMD;
import org.ow2.orchestra.designer.commands.DeployProcessCMD;
import org.ow2.orchestra.designer.commands.ExportProcessCMD;
import org.ow2.orchestra.designer.commands.FetchAdminProcessesCMD;
import org.ow2.orchestra.designer.commands.FetchUsersCMD;
import org.ow2.orchestra.designer.commands.FetchWsdlNavigatorModelCMD;
import org.ow2.orchestra.designer.commands.GetUsernameCMD;
import org.ow2.orchestra.designer.commands.NotificationCMD;
import org.ow2.orchestra.designer.commands.OpenProcessCMD;
import org.ow2.orchestra.designer.commands.PingServerCMD;
import org.ow2.orchestra.designer.commands.RedoCMD;
import org.ow2.orchestra.designer.commands.SaveProcessCMD;
import org.ow2.orchestra.designer.commands.SelectProcessCMD;
import org.ow2.orchestra.designer.commands.UndoCMD;
import org.ow2.orchestra.designer.commands.UpdateAccessPermissionCMD;
import org.ow2.orchestra.designer.commands.dataTypes.AddDataTypesCMD;
import org.ow2.orchestra.designer.commands.dataTypes.DeleteDataTypesCMD;
import org.ow2.orchestra.designer.commands.dataTypes.UpdateDataTypesCMD;
import org.ow2.orchestra.designer.commands.editor.AddDiagramElementCMD;
import org.ow2.orchestra.designer.commands.editor.AddDiagramElementWithFlowCMD;
import org.ow2.orchestra.designer.commands.editor.CreateFlowCMD;
import org.ow2.orchestra.designer.commands.editor.DeleteSelectedDiagramElementsCMD;
import org.ow2.orchestra.designer.commands.editor.MoveSelectedElementsCMD;
import org.ow2.orchestra.designer.commands.editor.ReorderFlowCMD;
import org.ow2.orchestra.designer.commands.editor.SelectDiagramElementsCMD;
import org.ow2.orchestra.designer.commands.editor.UpdateElementPropertyCMD;
import org.ow2.orchestra.designer.commands.fileNavigator.DeleteFileNavigatorItemCMD;
import org.ow2.orchestra.designer.commands.fileNavigator.FetchFileNavigatorTreeModelCMD;
import org.ow2.orchestra.designer.commands.fileNavigator.NewProcessCMD;
import org.ow2.orchestra.designer.commands.fileNavigator.NewProjectCMD;
import org.ow2.orchestra.designer.events.AccessPropertiesEvent;
import org.ow2.orchestra.designer.events.DataTypesChangeEvent;
import org.ow2.orchestra.designer.events.DesignerToolBarAppEvent;
import org.ow2.orchestra.designer.events.DiagramElementsSelectionCoordinatesChangeEvent;
import org.ow2.orchestra.designer.events.DiagramElementsSelectionEvent;
import org.ow2.orchestra.designer.events.DiagramEvent;
import org.ow2.orchestra.designer.events.DiagramModificationEvent;
import org.ow2.orchestra.designer.events.ElementPropertyChangeEvent;
import org.ow2.orchestra.designer.events.FileNavigatorEvent;
import org.ow2.orchestra.designer.events.FlowEvent;
import org.ow2.orchestra.designer.events.FlowOrderingEvent;
import org.ow2.orchestra.designer.events.ListUserOfDesignerEvent;
import org.ow2.orchestra.designer.events.NotificationEvent;
import org.ow2.orchestra.designer.events.PingServerEvent;
import org.ow2.orchestra.designer.events.ProcessEditorEvent;
import org.ow2.orchestra.designer.events.ProcessEvent;
import org.ow2.orchestra.designer.events.UndoRedoEvent;
import org.ow2.orchestra.designer.events.WsdlNavigatorEvent;

public class ProcessController extends FrontController {
  /**
   * Singleton Pattern.
   */
  private static var instance:ProcessController;

  public static function getInstance():ProcessController {
    if (instance == null) {
      new ProcessController();
    }

    return instance;
  }

  public function ProcessController() {
    super();
    if (instance != null) {
      throw new CairngormError(CairngormMessageCodes.SINGLETON_EXCEPTION, "ProcessController");
    }

    //The mxml doesn't use the getInstance method so we have to adapt the singleton
    // pattern by setting the instance here.
    instance = this;

    //Open process file
    addCommand(FileNavigatorEvent.PROCESS_OPEN, OpenProcessCMD);

    //Close process
    addCommand(ProcessEvent.PROCESS_CLOSE, CloseProcessCMD);

    //Select process
    addCommand(ProcessEvent.PROCESS_SELECT, SelectProcessCMD);

    //Get WsdlNavigator content
    addCommand(WsdlNavigatorEvent.FETCH_MODEL, FetchWsdlNavigatorModelCMD);

    //Get FileNavigator content
    addCommand(FileNavigatorEvent.FETCH_MODEL, FetchFileNavigatorTreeModelCMD);

    // Delete a projet or a process
    addCommand(FileNavigatorEvent.ITEM_DELETE, DeleteFileNavigatorItemCMD);

    // Add a new project
    addCommand(FileNavigatorEvent.PROJECT_CREATE, NewProjectCMD);

    // Add a new process
    addCommand(FileNavigatorEvent.PROCESS_CREATE, NewProcessCMD);

    // delete selected process elements
    addCommand(DiagramEvent.SELECTED_DIAGRAM_ELEMENTS_DELETE, DeleteSelectedDiagramElementsCMD);

    // add process element
    addCommand(DiagramModificationEvent.DIAGRAM_ELEMENT_ADD, AddDiagramElementCMD);

    // add process element with sequenceFlow
    addCommand(DiagramModificationEvent.DIAGRAM_ELEMENT_ADD_WITH_SEQUENCE, AddDiagramElementWithFlowCMD);

    // select process elements
    addCommand(DiagramElementsSelectionEvent.DIAGRAM_ELEMENTS_SELECT, SelectDiagramElementsCMD);

    // change a property on a process element
    addCommand(ElementPropertyChangeEvent.PROPERTY_CHANGED, UpdateElementPropertyCMD);

    // change a property on a process element
    addCommand(DiagramElementsSelectionCoordinatesChangeEvent.COORDINATES_CHANGE, MoveSelectedElementsCMD);

    // add a sequenceFlow
    addCommand(FlowEvent.FLOW_CREATION, CreateFlowCMD);

    // save a given list of process editor models.
    addCommand(ProcessEditorEvent.SAVE_MODELS, SaveProcessCMD);

    // export a given list of process editor models.
    addCommand(ProcessEditorEvent.EXPORT_MODELS, ExportProcessCMD);

    // deploy a given list of process editor models.
    addCommand(ProcessEditorEvent.DEPLOY_MODELS, DeployProcessCMD);

    // undo
    addCommand(UndoRedoEvent.UNDO, UndoCMD);
    // redo
    addCommand(UndoRedoEvent.REDO, RedoCMD);


    //add username
    addCommand(DesignerToolBarAppEvent.ADD_USERNAME, GetUsernameCMD);

    // confirm the modification of access permission
    addCommand(AccessPropertiesEvent.ACCESS_PERMISSIONS_CHANGED, UpdateAccessPermissionCMD);
    // recover the list of users designer
    addCommand(ListUserOfDesignerEvent.LIST_USERS_OF_DESIGNER, FetchUsersCMD);
    // change process
    addCommand(AccessPropertiesEvent.PROCESS_CHANGED, ChangeAccessSettingsProcessCMD);

    addCommand(AccessPropertiesEvent.FETCH_ADMIN_PROCESSES, FetchAdminProcessesCMD);

    // refresh token
    addCommand(PingServerEvent.PING_SERVER, PingServerCMD);

    //Notifications messages handling
    addCommand(NotificationEvent.ADD_NOTIFICATION, NotificationCMD);
    addCommand(NotificationEvent.REMOVE_NOTIFICATION, NotificationCMD);
    addCommand(NotificationEvent.CLEAR_NOTIFICATIONS, NotificationCMD);

    // change dataTypes for Event
    addCommand(DataTypesChangeEvent.DATA_TYPE_ADD, AddDataTypesCMD);
    addCommand(DataTypesChangeEvent.DATA_TYPE_DELETE, DeleteDataTypesCMD);
    addCommand(DataTypesChangeEvent.DATA_TYPE_UPDATE, UpdateDataTypesCMD);

    // Change flow order in data mapping
    addCommand(FlowOrderingEvent.REORDER, ReorderFlowCMD);
  }

}
}
